#include "orbit.hpp"

#include "astro/stateVectorIndices.hpp"
#include "astro/orbitalElementConversions.hpp"

Orbit::Orbit(int vertexCount)
{
    const float pi = 3.14159265359;

    std::vector<float> keplerianElements(6);
    keplerianElements[astro::semiMajorAxisIndex] = .75;
    keplerianElements[astro::eccentricityIndex] = .1;
    keplerianElements[astro::inclinationIndex] = pi / 2.0 + 0.1;
    keplerianElements[astro::argumentOfPeriapsisIndex] = 0;
    keplerianElements[astro::longitudeOfAscendingNodeIndex] = 0;

    for (int i = 0; i < vertexCount; i++)
    {
        float a = (float)i / (float)vertexCount;
        a *= 2.0 * pi;

        keplerianElements[astro::trueAnomalyIndex] = a;

        std::vector<float> cartesian = astro::convertKeplerianToCartesianElements(
            keplerianElements,
            1.0);

        _vertices.push_back(cartesian[astro::xPositionIndex]);
        _vertices.push_back(cartesian[astro::yPositionIndex]);
        _vertices.push_back(cartesian[astro::zPositionIndex]);
    }

    glGenVertexArrays(1, &_vao);
    glGenBuffers(1, &_vbo);

    glBindVertexArray(_vao);

    glBindBuffer(GL_ARRAY_BUFFER, _vbo);
    size_t vboBufferSize = _vertices.size() * sizeof(float);
    glBufferData(GL_ARRAY_BUFFER, vboBufferSize, &_vertices[0], GL_STATIC_DRAW);

    glEnableVertexAttribArray(0);
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*)0);

    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindVertexArray(0);
}

void Orbit::render()
{
    glBindVertexArray(_vao);
    glBindBuffer(GL_ARRAY_BUFFER, _vbo);

    glDrawArrays(GL_LINE_LOOP, 0, _vertices.size() / 3);
}

Orbit::~Orbit()
{
    glDeleteVertexArrays(1, &_vao);
    glDeleteBuffers(1, &_vbo);
}

